package	jp.co.sds_corp.ratemixer.fx.source;

import java.math.BigDecimal;
import java.util.Iterator;
import java.util.Map;

import jp.co.sds_corp.ratemixer.fx.FxRate;
import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* ダミーのレートを送信します。
* このクラスはテスト用です。
<PRE>
&lt;Source name="Dummy" class="jp.co.sds_corp.ratemixer.fx.source.DummySource"&gt;
	&lt;interval&gt;3&lt;/interval&gt;
	&lt;start&gt;70&lt;/start&gt;
	&lt;increment&gt;5&lt;/increment&gt;
	&lt;term&gt;SP&lt;/term&gt;
&lt;/Source&gt;
</PRE>
*/
public class DummySource extends FxSource
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	int			interval;
	BigDecimal	rate;
	BigDecimal	inc;
	String		term;
	transient ServiceThread thr;

	/**
	* 指定された初期情報でDummySourceを初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		String var;
		var = config.get( "interval", "5" );
		interval = Integer.valueOf( var ).intValue() * 1000;
		var = config.get( "start", "100.00" );
		rate = new BigDecimal( var );
		var = config.get( "increment", "1.00" );
		inc = new BigDecimal( var );
		term = config.get( "term", null );
	}

	/**
	* このサービスを開始します。
	* 裏Threadを起動し、指定された秒数間隔でダミーのレートを送信します。
	*/
	public void start() {
		thr = new ServiceThread();
		thr.setName( getName() );
		thr.start();
		setInterrupt( false );
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		setInterrupt( true );
		thr.interrupt();
		try {
			thr.join();
		} catch ( Exception ex ) {
		}
		logger.debug( this, "stopped." );
	}

	private void sendDummy( String key, FxRate oldRate ) throws Exception {
		FxRate newRate = new FxRate( oldRate );
		if ( oldRate.isInit() || oldRate.getBid() == null ) {
			newRate.setBid( rate );
			newRate.setAsk( rate );
		} else {
			newRate.setBid( oldRate.getBid().add( inc ));
			newRate.setAsk( oldRate.getAsk().add( inc ).add( inc ));
		}
		if ( oldRate.isInit() ) {
			fireStatusChange( new StatusChangeEvent( this,
									StatusChangeEvent.STAT_INIT,
									StatusChangeEvent.STAT_RUN,
									"start" ));
			oldRate.setInit( false );
		}
		fireReceive( newRate );
		mapRate.put( key, newRate );
	}

	private class ServiceThread extends Thread
	{
		public ServiceThread() {
		}

		public void run() {
			Map<String,FxRate> mapRate = getRateMap();
			try {
				Iterator<String> itr = mapRate.keySet().iterator();
				for ( ;; ) {
					if ( DummySource.this.isInterrupted()) {
						break;
					}

					if ( !itr.hasNext()) {
						itr = mapRate.keySet().iterator();
					}
					String key = (String )itr.next();
					FxRate rate = (FxRate )mapRate.get( key );
					if ( term != null && !term.equals( rate.getTerm() )) {
						continue;
					}
					sendDummy( key, rate );

					try {
						Thread.sleep( interval );
					} catch ( Exception ex ) {
					}
				}
			} catch ( Exception ex ) {
				logger.error( this, "error", ex );
			}
		}
	}

}
